/*
 *   PCM time-domain equalizer
 *
 *   Copyright (C) 2002  Felipe Rivera <liebremx at users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: eq.c,v 1.3 2002/08/31 18:55:33 felipe Exp $
 */

#include "eq.h"

float gain[EQ_MAX_BANDS][EQ_CHANNELS] __attribute__((aligned));
float preamp[EQ_CHANNELS] __attribute__((aligned));
static gboolean on = FALSE;
static void init(void);
static void cleanup(void);

EffectPlugin eq_ep =
{
	NULL,
	NULL,
	NULL, /* Description */
	init,
	cleanup,
	about,
	eq_configuration_window,
	mod_samples,
	NULL
};


EffectPlugin *get_eplugin_info(void)
{
	eq_ep.description = g_strdup_printf("EQ Plugin %s", VERSION);
	return &eq_ep;
}

static void init(void)
{
	/* Init the GUI first to load the configuration values */
	init_gui();
	init_iir();
	
	if (cfg.use_eplugins && get_current_effect_plugin() &&
		get_current_effect_plugin()->mod_samples == mod_samples)
	{
		on = cfg.use_eplugins;
		/* show_gui(); */
	}
}

static void cleanup(void)
{
	clean_gui();
}

void onswitch(gboolean onstatus)
{
	on = onstatus;
}

void set_eq_value(float value, gint index, gint chn)
{
	/* Map the gain and preamp values */
	if (index >= 0)
		gain[index][chn] = 0.0299995 * value + 
							0.001 * value * value;
	else
		preamp[chn] = 0.5 + 0.025 * value;
}

/* 
 * The main stuff
 */
int mod_samples(gpointer * d, gint length, AFormat afmt, gint srate, gint nch)
{
	/* Support all this later */
	if (!on || !(afmt == FMT_S16_NE ||
	      (afmt == FMT_S16_LE && G_BYTE_ORDER == G_LITTLE_ENDIAN) ||
	      (afmt == FMT_S16_BE && G_BYTE_ORDER == G_BIG_ENDIAN)))
		return length;
	return iir(d, length);
}
